@@ -273,18 +273,7 @@ typedef struct {
 
 R_API void r_core_gadget_free(RCoreGadget *g);
 
-typedef struct r_core_tasks_t {
-	int task_id_next;
-	RList *tasks;
-	RList *tasks_queue;
-	RList *oneshot_queue;
-	int oneshots_enqueued;
-	struct r_core_task_t *current_task;
-	struct r_core_task_t *main_task;
-	RThreadLock *lock;
-	int tasks_running;
-	bool oneshot_running;
-} RCoreTaskScheduler;
+#include "r_core_task.h"
 
 typedef struct r_core_project_t {
 	char *name;
@@ -434,11 +423,11 @@ struct r_core_t {
 	RThreadLock *lock;
 	bool in_log_process; // false;
 	RMainCallback r_main_radare2;
-	// int (*r_main_radare2)(int argc, char **argv);
 	int (*r_main_rafind2)(int argc, const char **argv);
 	int (*r_main_ravc2)(int argc, const char **argv);
 	int (*r_main_r2pm)(int argc, const char **argv);
 	int (*r_main_radiff2)(int argc, const char **argv);
+	int (*r_main_rafs2)(int argc, const char **argv);
 	int (*r_main_rabin2)(int argc, const char **argv);
 	int (*r_main_rarun2)(int argc, const char **argv);
 	int (*r_main_ragg2)(int argc, const char **argv);
@@ -554,7 +543,10 @@ R_API void r_core_seek_previous(RCore *core, const char *type);
 R_API void r_core_seek_next(RCore *core, const char *type);
 R_API int r_core_seek_align(RCore *core, ut64 align, int count);
 R_API void r_core_arch_bits_at(RCore *core, ut64 addr, R_OUT int * R_NULLABLE bits, R_OUT R_BORROW const char ** R_NULLABLE arch);
+R_API void r_core_prompt_format_help(RCore *core);
+R_API char *r_core_prompt_format(RCore *core, const char *fmt);
 R_API void r_core_seek_arch_bits(RCore *core, ut64 addr);
+R_API ut8 *r_core_readblock(RCore *core, ut64 size);
 R_API int r_core_block_read(RCore *core);
 R_API int r_core_block_size(RCore *core, int bsize);
 R_API int r_core_seek_size(RCore *core, ut64 addr, int bsize);
@@ -607,6 +599,7 @@ R_API void r_core_anal_inflags(RCore *core, const char *glob);
 R_API bool cmd_anal_objc(RCore *core, const char *input, bool auto_anal);
 R_API void r_core_anal_cc_init(RCore *core);
 R_API void r_core_anal_paths(RCore *core, ut64 from, ut64 to, bool followCalls, int followDepth, bool is_json);
+R_API void r_core_anal_esil_function(RCore *core, ut64 addr); /// TODO: better name and move to anal
 
 R_API int r_core_list_io(RCore *core, const char *name, int mode);
 R_API void r_core_list_lang(RCore *core, int mode);
@@ -762,7 +755,6 @@ R_API RList *r_core_asm_back_disassemble_instr(RCore *core, ut64 addr, int len,
 R_API RList *r_core_asm_back_disassemble_byte(RCore *core, ut64 addr, int len, ut32 hit_count, ut32 extra_padding);
 R_API ut32 r_core_asm_bwdis_len(RCore* core, int* len, ut64* start_addr, ut32 l);
 
-
 enum r_pdu_condition_t {
 	//pdu_esil,
 	pdu_instruction,
@@ -967,10 +959,10 @@ R_API RTable *r_core_table_new(RCore *core, const char *title);
 /* help */
 R_API void r_core_cmd_help(const RCore *core, RCoreHelpMessage help);
 R_API void r_core_cmd_help_json(const RCore *core, RCoreHelpMessage help);
-R_API void r_core_cmd_help_match(const RCore *core, RCoreHelpMessage help, R_BORROW char * R_NONNULL cmd);
-R_API void r_core_cmd_help_contains(const RCore *core, RCoreHelpMessage help, R_BORROW char * R_NONNULL cmd);
-R_API void r_core_cmd_help_match_spec(const RCore *core, const char * const help[], R_BORROW char * R_NONNULL cmd, char spec);
-R_API void r_core_cmd_help_contains_spec(const RCore *core, const char * const help[], R_BORROW char * R_NONNULL cmd, char spec);
+R_API int r_core_cmd_help_match(const RCore *core, RCoreHelpMessage help, const char * R_NONNULL cmd);
+R_API void r_core_cmd_help_match_spec(const RCore *core, const char * const help[], const char * R_NONNULL cmd, char spec);
+R_API void r_core_cmd_help_contains(const RCore *core, RCoreHelpMessage help, const char * R_NONNULL cmd);
+R_API void r_core_cmd_help_contains_spec(const RCore *core, const char * const help[], const char * R_NONNULL cmd, char spec);
 
 /* anal stats */
 
@@ -1009,67 +1001,10 @@ R_API void cmd_agfb(RCore *core);
 R_API void cmd_agfb2(RCore *core, const char *s);
 R_API void cmd_agfb3(RCore *core, const char *s, int x, int y);
 
-/* tasks */
-
-typedef void (*RCoreTaskCallback)(void *user, char *out);
-
-typedef enum {
-	R_CORE_TASK_STATE_BEFORE_START,
-	R_CORE_TASK_STATE_RUNNING,
-	R_CORE_TASK_STATE_SLEEPING,
-	R_CORE_TASK_STATE_DONE
-} RTaskState;
-
-typedef struct r_core_task_t {
-	int id;
-	RTaskState state;
-	bool transient; // delete when finished
-	int refcount;
-	RThreadSemaphore *running_sem;
-	void *user;
-	RCore *core;
-	bool dispatched;
-	RThreadCond *dispatch_cond;
-	RThreadLock *dispatch_lock;
-	RThread *thread;
-	char *cmd;
-	char *res;
-	bool cmd_log;
-	RConsContext *cons_context;
-	RCoreTaskCallback cb;
-} RCoreTask;
-
-typedef void (*RCoreTaskOneShot)(void *);
-
 R_API void r_print_list(RCore *core, const char *input);
 R_API void r_core_echo(RCore *core, const char *msg);
 R_API RTable *r_core_table(RCore *core, const char *name);
 
-R_API void r_core_task_scheduler_init(RCoreTaskScheduler *tasks, RCore *core);
-R_API void r_core_task_scheduler_fini(RCoreTaskScheduler *tasks);
-R_API RCoreTask *r_core_task_get(RCoreTaskScheduler *scheduler, int id);
-R_API RCoreTask *r_core_task_get_incref(RCoreTaskScheduler *scheduler, int id);
-R_API void r_core_task_print(RCore *core, RCoreTask *task, PJ *pj, int mode);
-R_API void r_core_task_list(RCore *core, int mode);
-R_API int r_core_task_running_tasks_count(RCoreTaskScheduler *scheduler);
-R_API const char *r_core_task_status(RCoreTask *task);
-R_API RCoreTask *r_core_task_new(RCore *core, bool create_cons, const char *cmd, RCoreTaskCallback cb, void *user);
-R_API void r_core_task_incref(RCoreTask *task);
-R_API void r_core_task_decref(RCoreTask *task);
-R_API void r_core_task_enqueue(RCoreTaskScheduler *scheduler, RCoreTask *task);
-R_API void r_core_task_enqueue_oneshot(RCoreTaskScheduler *scheduler, RCoreTaskOneShot func, void *user);
-R_API int r_core_task_run_sync(RCoreTaskScheduler *scheduler, RCoreTask *task);
-R_API void r_core_task_sync_begin(RCoreTaskScheduler *scheduler);
-R_API void r_core_task_sync_end(RCoreTaskScheduler *scheduler);
-R_API void r_core_task_yield(RCoreTaskScheduler *scheduler);
-R_API void r_core_task_sleep_begin(RCoreTask *task);
-R_API void r_core_task_sleep_end(RCoreTask *task);
-R_API void r_core_task_break(RCoreTaskScheduler *scheduler, int id);
-R_API void r_core_task_break_all(RCoreTaskScheduler *scheduler);
-R_API int r_core_task_del(RCoreTaskScheduler *scheduler, int id);
-R_API void r_core_task_del_all_done(RCoreTaskScheduler *scheduler);
-R_API RCoreTask *r_core_task_self(RCoreTaskScheduler *scheduler);
-R_API void r_core_task_join(RCoreTaskScheduler *scheduler, RCoreTask *current, int id);
 typedef void (*inRangeCb) (RCore *core, ut64 from, ut64 to, int vsize, void *cb_user);
 R_IPI int r_core_search_value_in_range(RCore *core, bool relative, RInterval search_itv, ut64 vmin, ut64 vmax, int vsize, inRangeCb cb, void *cb_user);
 
