@@ -1,4 +1,4 @@
-/* radare2 - LGPL - Copyright 2017-2024 - condret, pancake */
+/* radare2 - LGPL - Copyright 2017-2025 - condret, pancake */
 
 #ifndef R2_IO_H
 #define R2_IO_H
@@ -13,6 +13,7 @@
 #define R_IO_SEEK_SET 0
 #define R_IO_SEEK_CUR 1
 #define R_IO_SEEK_END 2
+#define R_IO_SEEK_HOLE 3
 
 #define R_IO_UNDOS 64
 
@@ -130,24 +131,23 @@ typedef struct r_io_cache_t {
 // -io-cache-
 
 typedef struct r_io_t {
-	struct r_io_desc_t *desc; // XXX R2_590 - deprecate... we should use only the fd integer, not hold a weak pointer
-	ut64 off;
+	struct r_io_desc_t *desc; // XXX R2_600 - deprecate... we should use only the fd integer, not hold a weak pointer
+	ut64 off; // XXX maybe deprecate this global seek? also it should be addr, not off R2_600
 	ut32 bank; // current bank
 	int bits;
 	int va;	// keep it as int, value can be 0, 1 or 2
 	bool ff;
 	ut8 Oxff; // which printable char to use instead of 0xff for unallocated bytes
 	size_t addrbytes; // XXX also available in RArchConfig.addrbytes
-	bool aslr;
 	bool autofd;
 	bool overlay;
 	// moved into cache.mode // ut32 cached; // uses R_PERM_RWX // wtf cache for exec?
 	bool cachemode; // write in cache all the read operations (EXPERIMENTAL)
 	ut32 p_cache; // uses 1, 2, 4.. probably R_PERM_RWX :D
 	ut64 mts; // map "timestamps", this sucks somehow
-	RIDStorage *files; // RIODescs accessible by their fd
-	RIDStorage *maps;  // RIOMaps accessible by their id
-	RIDStorage *banks; // RIOBanks accessible by their id
+	RIDStorage files; // RIODescs accessible by their fd
+	RIDStorage maps;  // RIOMaps accessible by their id
+	RIDStorage banks; // RIOBanks accessible by their id
 	RIOCache cache;
 	ut8 *write_mask;
 	int write_mask_len;
@@ -193,10 +193,9 @@ typedef struct {
 
 typedef struct r_io_plugin_t {
 	const RPluginMeta meta;
-	void *widget;
+	void *data; // kind of globals, used by rlang-io in this case
 	const char *uris;
 	int (*listener)(RIODesc *io);
-	bool (*init)(void);
 	bool isdbg;
 	// int (*is_file_opened)(RIO *io, RIODesc *fd, const char *);
 	char *(*system)(RIO *io, RIODesc *fd, const char *); // Rename to call? or cmd? unify with anal and core
@@ -224,6 +223,43 @@ typedef struct r_io_plugin_t {
 #define	R_IO_MAP_TIE_FLG_BACK	1		//ties a map so that it resizes when the desc resizes
 #define	R_IO_MAP_TIE_FLG_FORTH	(1 << 1)	//ties a map so that the desc resizes when the map resizes
 
+typedef enum {
+	R_IO_MAP_META_TYPE_NONE = 0,
+	R_IO_MAP_META_TYPE_HEAP, // heap memory
+	R_IO_MAP_META_TYPE_STACK, // program stack
+	R_IO_MAP_META_TYPE_MMAP, // mapped memory
+	R_IO_MAP_META_TYPE_MMIO, // mapped devices
+	R_IO_MAP_META_TYPE_DMA, // high speed hw data transfer
+	R_IO_MAP_META_TYPE_JIT, // just in time code
+	R_IO_MAP_META_TYPE_BSS, // block started symbol (zero paged memory)
+	R_IO_MAP_META_TYPE_SHARED, // ipc, etc
+	R_IO_MAP_META_TYPE_KERNEL, // VDSO, etc, text|data|buffers
+	R_IO_MAP_META_TYPE_GUARD, // surrounding stack for protections
+	R_IO_MAP_META_TYPE_NULL, // to catch null derefs
+	R_IO_MAP_META_TYPE_GPU, // graphics memory
+	R_IO_MAP_META_TYPE_TLS, // thread-local storage
+	R_IO_MAP_META_TYPE_BUFFER, // temporal
+	R_IO_MAP_META_TYPE_COW, // copy on write
+	R_IO_MAP_META_TYPE_PAGETABLES, // mmu settings
+	R_IO_MAP_META_TYPE_LAST
+} RIOMapMetaType;
+
+#define R_IO_MAP_META_FLAG_LAST 16
+typedef enum {
+	R_IO_MAP_META_FLAG_PAGED, // anything can be non-paged.. must be bitfield
+	R_IO_MAP_META_FLAG_PRIVATE, // private memory
+	R_IO_MAP_META_FLAG_PERSISTENT, // non volatile
+	R_IO_MAP_META_FLAG_ASLR, // randomizable
+	R_IO_MAP_META_FLAG_SWAP, // swappage to disk
+	R_IO_MAP_META_FLAG_DEP, // same as W^X
+	R_IO_MAP_META_FLAG_ENCLAVE, // protected by a secure enclave
+	R_IO_MAP_META_FLAG_COMPRESSED, // compressed memory
+	R_IO_MAP_META_FLAG_ENCRYPTED, // cryptographically secure
+	R_IO_MAP_META_FLAG_LARGE, // different alignment for big data
+	R_IO_MAP_META_FLAG_SYSTEM, // frameworks, dyldcache, ..
+	R_IO_MAP_META_FLAG_LIBRARY, // maybe the same of system?
+} RIOMapMetaFlags;
+
 typedef struct r_io_map_t {
 	int fd;
 	int perm;
@@ -234,6 +270,7 @@ typedef struct r_io_map_t {
 	RRBTree *overlay;
 	char *name;
 	ut32 tie_flags;
+	ut32 meta; // metadata attributes
 } RIOMap;
 
 typedef struct r_io_map_ref_t {
@@ -284,7 +321,7 @@ typedef bool (*RIOFdClose)(RIO *io, int fd);
 typedef ut64 (*RIOFdSeek)(RIO *io, int fd, ut64 addr, int whence);
 typedef ut64 (*RIOFdSize)(RIO *io, int fd);
 typedef bool (*RIOFdResize)(RIO *io, int fd, ut64 newsize);
-typedef ut64 (*RIOP2V)(RIO *io, ut64 pa);
+typedef bool (*RIOP2V)(RIO *io, ut64 p, ut64 *v);
 typedef ut64 (*RIOV2P)(RIO *io, ut64 va);
 typedef int (*RIOFdRead)(RIO *io, int fd, ut8 *buf, int len);
 typedef int (*RIOFdWrite)(RIO *io, int fd, const ut8 *buf, int len);
@@ -296,6 +333,7 @@ typedef RList *(*RIOFdGetMap)(RIO *io, int fd);
 typedef bool (*RIOFdRemap)(RIO *io, int fd, ut64 addr);
 typedef bool (*RIOIsValidOff)(RIO *io, ut64 addr, int hasperm);
 typedef RIOBank *(*RIOBankGet)(RIO *io, ut32 bankid);
+typedef bool (*RIOBankUse)(RIO *io, ut32 bankid);
 typedef RIOMap *(*RIOMapGet)(RIO *io, ut32 id);
 typedef RIOMap *(*RIOMapGetAt)(RIO *io, ut64 addr);
 typedef RIOMap *(*RIOMapGetPaddr)(RIO *io, ut64 paddr);
@@ -335,6 +373,7 @@ typedef struct r_io_bind_t {
 	RIOIsValidOff is_valid_offset;
 	RIOAddrIsMapped addr_is_mapped;
 	RIOBankGet bank_get;
+	RIOBankUse bank_use;
 	RIOMapGet map_get;
 	RIOMapGetAt map_get_at;
 	RIOMapGetPaddr map_get_paddr;
@@ -378,13 +417,15 @@ R_API void r_io_map_read_from_overlay(RIOMap *map, ut64 addr, ut8 *buf, int len)
 R_API bool r_io_map_write_to_overlay(RIOMap *map, ut64 addr, const ut8 *buf, int len);
 R_IPI bool io_map_get_overlay_intersects(RIOMap *map, RQueue *q, ut64 addr, int len);
 R_API void r_io_map_drain_overlay(RIOMap *map);
+typedef void (*RIOMapOverlayForeach)(RInterval itv, const ut8 *data,  void *user);
+R_API void r_io_map_overlay_foreach(RIOMap *map, RIOMapOverlayForeach cb, void *user);
 
 // next free address to place a map.. maybe just unify
 R_API bool r_io_map_locate(RIO *io, ut64 *addr, const ut64 size, ut64 load_align);
 
 // p2v/v2p
 
-R_API ut64 r_io_p2v(RIO *io, ut64 pa);
+R_API bool r_io_p2v(RIO *io, ut64 p, ut64 *v);
 R_API ut64 r_io_v2p(RIO *io, ut64 va);
 
 //io_submap.c
@@ -415,6 +456,8 @@ R_API RIOMap *r_io_bank_get_map_at(RIO *io, const ut32 bankid, const ut64 addr);
 R_API bool r_io_bank_read_at(RIO *io, const ut32 bankid, ut64 addr, ut8 *buf, int len);
 R_API bool r_io_bank_write_at(RIO *io, const ut32 bankid, ut64 addr, const ut8 *buf, int len);
 R_API bool r_io_bank_write_to_overlay_at(RIO *io, const ut32 bankid, ut64 addr, const ut8 *buf, int len);
+typedef void (*RIOOverlayForeach)(RInterval itv, const ut8 *m_data, const ut8 *o_data, void *user);
+R_API void r_io_bank_overlay_foreach(RIO *io, const ut32 bankid, RIOOverlayForeach cb, void *user);
 R_API int r_io_bank_read_from_submap_at(RIO *io, const ut32 bankid, ut64 addr, ut8 *buf, int len);
 R_API int r_io_bank_write_to_submap_at(RIO *io, const ut32 bankid, ut64 addr, const ut8 *buf, int len);
 R_API void r_io_bank_drain(RIO *io, const ut32 bankid);
@@ -459,8 +502,6 @@ R_API void r_io_free(RIO *io);
 R_API bool r_io_plugin_init(RIO *io);
 R_API bool r_io_plugin_add(RIO *io, RIOPlugin *plugin);
 R_API bool r_io_plugin_remove(RIO *io, RIOPlugin *plugin);
-R_API int r_io_plugin_list(RIO *io);
-R_API int r_io_plugin_list_json(RIO *io);
 R_API int r_io_plugin_read(RIODesc *desc, ut8 *buf, int len);
 R_API int r_io_plugin_write(RIODesc *desc, const ut8 *buf, int len);
 R_API int r_io_plugin_read_at(RIODesc *desc, ut64 addr, ut8 *buf, int len);
@@ -527,7 +568,7 @@ R_IPI void r_io_desc_fini(RIO *io);
 /* io/cache.c */
 R_API void r_io_cache_init(RIO *io);
 R_API void r_io_cache_fini(RIO *io);
-R_API void r_io_cache_list(RIO *io, int rad, bool many);
+R_API char *r_io_cache_list(RIO *io, int rad, bool many);
 R_API bool r_io_cache_empty(RIO *io);
 R_API void r_io_cache_reset(RIO *io);
 R_API bool r_io_cache_at(RIO *io, ut64 addr);
@@ -580,9 +621,9 @@ R_API int r_io_fd_get_prev(RIO *io, int fd);
 R_API int r_io_fd_get_highest(RIO *io);
 R_API int r_io_fd_get_lowest(RIO *io);
 
-
-#define r_io_range_new()	R_NEW0(RIORange)
-#define r_io_range_free(x)	free(x)
+R_API bool r_io_map_setattr_fromstring(RIOMap *map, const char *s);
+R_API bool r_io_map_setattr(RIOMap *map, ut32 type, ut32 flags);
+R_API char *r_io_map_getattr(RIOMap *map);
 
 /* io/ioutils.c */
 R_API bool r_io_is_valid_offset(RIO *io, ut64 offset, int hasperm);
@@ -637,6 +678,7 @@ extern RIOPlugin r_io_plugin_xalz;
 extern RIOPlugin r_io_plugin_reg;
 extern RIOPlugin r_io_plugin_treebuf;
 extern RIOPlugin r_io_plugin_sysgdb;
+extern RIOPlugin r_io_plugin_pvm;
 extern RIOPlugin r_io_plugin_serial;
 extern RIOPlugin r_io_plugin_cyclic;
 extern RIOPlugin r_io_plugin_uf2;
